# ImgAssist Python Toolkit

## Model Generation Tool
* Generation and performance evaluation of ImgAssist models

### Development Environment
* Ubuntu Linux

### Runtime Environment
* Python 3.9.1, standard libraries
* Additional Python packages: Matplotlib 3.3.3, NumPy 1.19.5, OpenCV 4.5.1, Scikit-learn 0.24.0, Torch 1.7.1+cu110, Torchvision 0.8.2+cu110

### Run Instructions
* Model normal mode: modelGenerationTool.py input_params.json
	* This will create the input folders, generate a model and then test it
* Model init mode: modelGenerationTool.py --init input_params.json
	* This will only create the input folders and initialize them with chosen patches
* Model training mode: modelGenerationTool.py --train input_params.json
	* This will only generate an output model, it assumes the input folders already exist
* Model testing mode: modelGenerationTool.py --test input_params.json
	* This will only test an output model, it assumes the input folders already exist and a model has been generated

### Input Parameters
* tool:				 Target tool
* version:			 Target tool version of input file
* modelFileName: 		 Output name of trained model
* numEpochs:			 Maximum number of training epochs
* epochsNoImprove: 		 Number of epochs without improvement in loss function to interrupt training
* inputDataFolder: 		 Top level folder with input patches organized by unique case IDs
* trainingDataFolder: 		 Top level folder of model generation data pulled from inputDataFolder
* destinationFolder: 	    	 Folder to save the trained model, datasets used and performance report
* suspiciousClassScaleFactor:  Sets the ratio between positive and negative patches
* weightReductionCoefficient:  Adjusts the weighting of the neural network
* classificationConfidenceStep: Sets the step size used in test mode
* suspiciousTrainingCaseList:    List of suspicious training scans
* nonSuspiciousTrainingCaseList: List of non-suspicious training scans
* suspiciousValidationCaseList:  List of suspicious validation scans
* nonSuspiciousTrainingCaseList: List of non-suspicious validation scans
* testCaseList: 		 List of test scans


## Patch Generation Tool
* Generation training and testing patches for ImgAssist

### Development Environment
* Ubuntu Linux

### Runtime Environment
* Python 3.9.1, standard libraries
* Additional Python packages: Imageio 2.9.0, NumPy 1.19.5, Scikit-image 0.18.1

### Run Instructions
* patchGenerationTool.py input_params.json

### Input Parameters
* tool:				Target tool
* version:			Target tool version of input file
* inputFolder: 			Top level folder with cases from which patches will be extracted
* destinationFolder: 	    	Folder to save output patches
* caseList: 		       	List of cases from which patches will be extracted


## Processing Tool
* Application of ImgAssist to WF_OCT data

### Development Environment
* Ubuntu Linux

### Runtime Environment
* Python 3.9.1, standard libraries
* Additional Python packages: Imageio 2.9.0, NumPy 1.19.5, OpenCV 4.5.1, Scikit-image 0.18.1, Scikit-learn 0.24.0, Torch 1.7.1+cu110, Torchvision 0.8.2+cu110
* Additional software: ImageMagick 6.8.10-23

### Run Instructions
* processingTool.py input_params.json

### Input Parameters
* tool:				Target tool
* version:			Target tool version of input file
* modelFolder:			Folder with ImgAssist model
* modelFileName: 		Name of ImgAssist model file
* classificationConfidence:	Confidence threshold
* inputFolder: 			Top level folder with cases to be processed
* destinationFolder: 	    	Folder to save findings and corresponding image patches
* caseList: 		       	List of cases to be processed

### Tool Deployment Instructions
* Checkout the desired branch
* Tag the branch with version if releasing in Arena
* Create a new or clean an existing deployment folder
* Run deploy script with appropriate parameters
* Zip deployment folder
* Rename the zip file according to Perimeter naming conventions for controlled items (see QD-011)
	* Naming should be similar to as follows: (Item Number) rev (Revision)_(Item Name) (i.e. 520-00025_rev_B_ModelGenerationTool.zip)
	* Refer to QD-011 for latest in naming convention

### Deployment Script Run Instructions
* deploy.sh \<option> \<deployment folder>

### Options
* 1:	Deployment of Model Generation Tool
* 2:	Deployment of Patch Generation Tool
* 3: 	Deployment of Processing Tool
* a:	Deployment of complete toolkit
