# Standard packages

# Additional packages
import cv2
import torch
from torchvision import transforms

# Local modules
from commonTools import common_defs
from commonTools.imgassist import Resize
from processingTool import constants, stats


# Apply ImgAssist model to a B-scan
def analyze_data(net, confidence, patch_data, labels, bscan, yw, pixel_width, stride_x, offset_y, index_of_refraction):
    result = []
    result_patches = []
    result_names = []

    data_transform = transforms.Compose([
        Resize((constants.patchSize, constants.patchSize)),
        transforms.ToTensor(),
        transforms.Normalize([constants.dataMean], [constants.dataStdDev])
    ])

    t = torch.tensor([confidence]).cuda()
    imlist = []

    for x in range(patch_data.shape[1]):
        image = data_transform(patch_data[0][x])
        imlist.append(image.view(1, 1, constants.patchSize, constants.patchSize))

    inputs = torch.cat(imlist).cuda()
    outputs = net(inputs)
    # convert outputs to probabilities
    outputs_softmax = torch.softmax(outputs, dim=1)
    prob, predicted = torch.max(outputs_softmax.data, 1)
    mask = (prob >= t).long() * 1
    predicted *= mask
    predicted_labels = predicted.cpu().numpy()
    patch_conf = prob.cpu().numpy()

    for i in range(len(labels)):
        if predicted_labels[i] == common_defs.suspicious:
            xw = i * stride_x * pixel_width
            zw = constants.verticalOffsetMm / index_of_refraction
            result.append([xw, yw, zw, labels[i], patch_conf[i]])
            thumbnail = cv2.resize(patch_data[0][i], (constants.thumbnailSize, constants.thumbnailSize))

            if predicted_labels[i] == common_defs.suspicious and labels[i] > 0:
                stats.num_true_positives += 1
            elif labels[i] == 0:
                stats.num_false_positives += 1

            result_patches.append(thumbnail)
            patch_name = '%04d_%04d_%04d' % (bscan, i * stride_x, offset_y)
            result_names.append(patch_name)
        elif predicted_labels[i] == common_defs.nonSuspicious:
            if predicted_labels[i] == common_defs.nonSuspicious and labels[i] == 0:
                stats.num_true_negatives += 1
            elif labels[i] > 0:
                stats.num_false_negatives += 1

    return result, result_patches, result_names
